/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_CPU_OPERATOR_DIVIMPL_FORWARD_KERNEL_H_
#define AIDGE_CPU_OPERATOR_DIVIMPL_FORWARD_KERNEL_H_

#include "aidge/utils/Registrar.hpp"

#include "aidge/backend/cpu/operator/DivImpl.hpp"

namespace Aidge {
template <class I1, class I2, class O>
void DivImpl_cpu_forward_kernel(std::size_t input1Length,
                                     std::size_t input2Length,
                                     const void* input1_,
                                     const void* input2_,
                                     void* output_) {

    const I1* input_1 = static_cast<const I1*>(input1_);
    const I2* input_2 = static_cast<const I2*>(input2_);
    O* output = static_cast<O*>(output_);
    if (input2Length == input1Length)
    {
        for (std::size_t i = 0; i < input1Length; ++i) {
            output[i] = input_1[i] / input_2[i];
        }
    }
    else if (input2Length == 1)
    {
        for (std::size_t i = 0; i < input1Length; ++i) {
            output[i] = input_1[i] / input_2[0];
        }
    }
    else // input_2 is 1d and of size the number of channels of input_1
    {
        for (std::size_t i = 0; i < input1Length; ++i) {
            std::size_t channelIdx = i % input2Length;
            output[i] = input_1[i] / input_2[channelIdx];
        }
    }
}

namespace {
static Registrar<DivImplForward_cpu> registrarDivImplForward_cpu_Float32(
        {DataType::Float32, DataType::Float32, DataType::Float32},
        Aidge::DivImpl_cpu_forward_kernel<float, float, float>);
static Registrar<DivImplForward_cpu> registrarDivImplForward_cpu_Int32(
        {DataType::Int32, DataType::Int32, DataType::Int32},
        Aidge::DivImpl_cpu_forward_kernel<int, int, int>);
static Registrar<DivImplForward_cpu> registrarDivImplForward_cpu_Float64(
        {DataType::Float64, DataType::Float64, DataType::Float64},
        Aidge::DivImpl_cpu_forward_kernel<double, double, double>);
}  // namespace
}  // namespace Aidge

#endif /* AIDGE_CPU_OPERATOR_DIVIMPL_FORWARD_KERNEL_H_ */
